# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinSupport library (derived from EventAggregatorSupport)

    @copyright: 2008, 2009, 2010, 2011, 2012 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2005-2008 MoinMoin:ThomasWaldmann.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from DateSupport import *
from MoinMoin.Page import Page
from MoinMoin import wikiutil
import re
import time

__version__ = "0.1"

# Content type parsing.

encoding_regexp_str = ur'(?P<content_type>[^\s;]*)(?:;\s*charset=(?P<encoding>[-A-Za-z0-9]+))?'
encoding_regexp = re.compile(encoding_regexp_str)

# Utility functions.

def getContentTypeAndEncoding(content_type):
    m = encoding_regexp.search(content_type)
    if m:
        return m.group("content_type"), m.group("encoding")
    else:
        return None, None

def int_or_none(x):
    if x is None:
        return x
    else:
        return int(x)

# Utility classes and associated functions.

class Form:

    """
    A wrapper preserving MoinMoin 1.8.x (and earlier) behaviour in a 1.9.x
    environment.
    """

    def __init__(self, form):
        self.form = form

    def has_key(self, name):
        return not not self.form.getlist(name)

    def get(self, name, default=None):
        values = self.form.getlist(name)
        if not values:
            return default
        else:
            return values

    def __getitem__(self, name):
        return self.form.getlist(name)

class ActionSupport:

    """
    Work around disruptive MoinMoin changes in 1.9, and also provide useful
    convenience methods.
    """

    def get_form(self):
        return get_form(self.request)

    def _get_selected(self, value, input_value):

        """
        Return the HTML attribute text indicating selection of an option (or
        otherwise) if 'value' matches 'input_value'.
        """

        return input_value is not None and value == input_value and 'selected="selected"' or ''

    def _get_selected_for_list(self, value, input_values):

        """
        Return the HTML attribute text indicating selection of an option (or
        otherwise) if 'value' matches one of the 'input_values'.
        """

        return value in input_values and 'selected="selected"' or ''

    def _get_input(self, form, name, default=None):

        """
        Return the input from 'form' having the given 'name', returning either
        the input converted to an integer or the given 'default' (optional, None
        if not specified).
        """

        value = form.get(name, [None])[0]
        if not value: # true if 0 obtained
            return default
        else:
            return int(value)

def get_form(request):

    "Work around disruptive MoinMoin changes in 1.9."

    if hasattr(request, "values"):
        return Form(request.values)
    else:
        return request.form

class send_headers_cls:

    """
    A wrapper to preserve MoinMoin 1.8.x (and earlier) request behaviour in a
    1.9.x environment.
    """

    def __init__(self, request):
        self.request = request

    def __call__(self, headers):
        for header in headers:
            parts = header.split(":")
            self.request.headers.add(parts[0], ":".join(parts[1:]))

def get_send_headers(request):

    "Return a function that can send response headers."

    if hasattr(request, "http_headers"):
        return request.http_headers
    elif hasattr(request, "emit_http_headers"):
        return request.emit_http_headers
    else:
        return send_headers_cls(request)

def escattr(s):
    return wikiutil.escape(s, 1)

def getPathInfo(request):
    if hasattr(request, "getPathinfo"):
        return request.getPathinfo()
    else:
        return request.path

# Page access functions.

def getPageURL(page):

    "Return the URL of the given 'page'."

    request = page.request
    return request.getQualifiedURL(page.url(request, relative=0))

def getFormat(page):

    "Get the format used on the given 'page'."

    return page.pi["format"]

def getMetadata(page):

    """
    Return a dictionary containing items describing for the given 'page' the
    page's "created" time, "last-modified" time, "sequence" (or revision number)
    and the "last-comment" made about the last edit.
    """

    request = page.request

    # Get the initial revision of the page.

    revisions = page.getRevList()
    event_page_initial = Page(request, page.page_name, rev=revisions[-1])

    # Get the created and last modified times.

    initial_revision = getPageRevision(event_page_initial)

    metadata = {}
    metadata["created"] = initial_revision["timestamp"]
    latest_revision = getPageRevision(page)
    metadata["last-modified"] = latest_revision["timestamp"]
    metadata["sequence"] = len(revisions) - 1
    metadata["last-comment"] = latest_revision["comment"]

    return metadata

def getPageRevision(page):

    "Return the revision details dictionary for the given 'page'."

    # From Page.edit_info...

    if hasattr(page, "editlog_entry"):
        line = page.editlog_entry()
    else:
        line = page._last_edited(page.request) # MoinMoin 1.5.x and 1.6.x

    # Similar to Page.mtime_usecs behaviour...

    if line:
        timestamp = line.ed_time_usecs
        mtime = wikiutil.version2timestamp(long(timestamp)) # must be long for py 2.2.x
        comment = line.comment
    else:
        mtime = 0
        comment = ""

    # Leave the time zone empty.

    return {"timestamp" : DateTime(time.gmtime(mtime)[:6] + (None,)), "comment" : comment}

# User interface functions.

def getParameter(request, name, default=None):

    """
    Using the given 'request', return the value of the parameter with the given
    'name', returning the optional 'default' (or None) if no value was supplied
    in the 'request'.
    """

    return get_form(request).get(name, [default])[0]

def getQualifiedParameter(request, prefix, argname, default=None):

    """
    Using the given 'request', 'prefix' and 'argname', retrieve the value of the
    qualified parameter, returning the optional 'default' (or None) if no value
    was supplied in the 'request'.
    """

    argname = getQualifiedParameterName(prefix, argname)
    return getParameter(request, argname, default)

def getQualifiedParameterName(prefix, argname):

    """
    Return the qualified parameter name using the given 'prefix' and 'argname'.
    """

    if prefix is None:
        return argname
    else:
        return "%s-%s" % (prefix, argname)

# Page-related functions.

def getPrettyPageName(page):

    "Return a nicely formatted title/name for the given 'page'."

    title = page.split_title(force=1)
    return getPrettyTitle(title)

def linkToPage(request, page, text, query_string=None):

    """
    Using 'request', return a link to 'page' with the given link 'text' and
    optional 'query_string'.
    """

    text = wikiutil.escape(text)
    return page.link_to_raw(request, text, query_string)

def linkToResource(url, request, text, query_string=None):

    """
    Using 'request', return a link to 'url' with the given link 'text' and
    optional 'query_string'.
    """

    if query_string:
        query_string = wikiutil.makeQueryString(query_string)
        url = "%s?%s" % (url, query_string)

    formatter = request.page and getattr(request.page, "formatter", None) or request.html_formatter

    output = []
    output.append(formatter.url(1, url))
    output.append(formatter.text(text))
    output.append(formatter.url(0))
    return "".join(output)

def getFullPageName(parent, title):

    """
    Return a full page name from the given 'parent' page (can be empty or None)
    and 'title' (a simple page name).
    """

    if parent:
        return "%s/%s" % (parent.rstrip("/"), title)
    else:
        return title

# vim: tabstop=4 expandtab shiftwidth=4
